@@ -2,7 +2,7 @@
 #
 #  Net::Server::INET - Net::Server personality
 #  
-#  $Id: INET.pm,v 1.17 2001/03/29 08:24:45 rhandom Exp $
+#  $Id: INET.pm,v 1.18 2001/05/01 06:41:14 rhandom Exp $
 #  
 #  Copyright (C) 2001, Paul T Seamons
 #                      paul@seamons.com
@@ -35,7 +35,23 @@ sub pre_bind {}
 sub bind {}
 
 ### connection is already accepted
-sub accept { 1 }
+sub accept { 
+  my $self = shift;
+  my $prop = $self->{server};
+
+  ### need to determine how to pass this
+#  $self->get_sock_protocol( *STDIN );
+
+  ### receive a udp packet
+  if( $prop->{udp_true} ){
+    $prop->{client}   = *STDIN;
+    $prop->{udp_peer} = STDIN->recv($prop->{udp_data},
+                                    $prop->{udp_packet_size},
+                                    $prop->{udp_packet_offset});
+  }
+
+  1;
+}
 
 ### accept only one connection per process
 sub done { 1 }
